/**
 * Copyright (c) 2009 Jeremy Stribling, Yair Sovran, Irene Zhang, 
 *                    Xavid Pretzer, Jinyang Li, Frans Kaashoek, Robert Morris,
 *     		      Massachusetts Institute of Technology,
 * 		      New York University
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */


#ifndef __WFS_CACHE_MEM_H__
#define __WFS_CACHE_MEM_H__ 1

#include "wfs_cache_blockfile.h"
#include <ext/hash_map>

using __gnu_cxx::hash_map;

struct wfs_cache_mem_key {
  
  wfs_file_id fid;
  wfs_version version;
  unsigned int block_no;
  
  wfs_cache_mem_key() {}
  wfs_cache_mem_key( wfs_file_id f, wfs_version v, unsigned int b ) :
    fid(f), version(v), block_no(b) {}
  
};

bool operator==( const wfs_cache_mem_key &a, const wfs_cache_mem_key &b );
bool operator!=( const wfs_cache_mem_key &a, const wfs_cache_mem_key &b );
bool operator<( const wfs_cache_mem_key &a, const wfs_cache_mem_key &b );

struct wfs_cache_mem_key_hash {
  std::size_t operator()( const wfs_cache_mem_key &a ) const {
    return a.fid ^ a.version.version_num ^ a.block_no;
  }
};

class wfs_cache_mem : public wfs_cache_blockfile {

 private:

  struct cache_mem_entry {
    wfs_cache_mem_key key;
    std::string buf;
    cache_mem_entry *prev;
    cache_mem_entry *next;

    cache_mem_entry() : prev(NULL), next(NULL) {}

  };

  int _cache_size;
  hash_map< wfs_cache_mem_key, cache_mem_entry *, wfs_cache_mem_key_hash > _cache;
  cache_mem_entry *_lru;
  cache_mem_entry *_mru;
  int _curr_size;

  pthread_mutex_t _frans_cachelock;

  void update_ru( cache_mem_entry *entry );

 public:

  wfs_cache_mem( std::string cache_dir, unsigned int cache_size );
  ~wfs_cache_mem();

  
  bool read_block( wfs_file_handle fh, wfs_version version, 
		   unsigned int block_no, std::string &buf );
  bool read_block( wfs_file_handle fh, wfs_version version, 
		   unsigned int block_no );
  void store_block( wfs_file_handle fh, wfs_version version, 
		    unsigned int block_no, std::string buf );

};

#endif // __WFS_CACHE_MEM_H__
