/**
 * Copyright (c) 2009 Jeremy Stribling, Yair Sovran, Irene Zhang, 
 *                    Xavid Pretzer, Jinyang Li, Frans Kaashoek, Robert Morris,
 *     		      Massachusetts Institute of Technology,
 * 		      New York University
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */


#ifndef __WFS_CACHELOC_H
#define __WFS_CACHELOC_H 1

#include <arpa/inet.h>
#include <vector>
#include <map>
#include <vivaldi.h>
#include <wfs_file.h>
#ifdef WFS_WITH_LIBSSH
#include <openssl/sha.h>
#endif

typedef struct cacheloc {
  uint32_t s_addr;
  uint16_t s_port;
  Coord c;
  std::vector<unsigned long long> off;
  std::vector<unsigned long long> n;
  uint32_t age;
  double when;
#ifdef WFS_WITH_LIBSSH
  struct checksum {
    unsigned char data[SHA256_DIGEST_LENGTH];
  };
  // Maps a block number to a checksum for that block
  std::map<unsigned,checksum> checksums;
#endif
} cacheloc_ent;

class wfs_cacheloc {

  public:
	 wfs_cacheloc();
	 ~wfs_cacheloc();

	 bool get_close_nodes(  wfs_file_id fid, wfs_version v, vivaldi *viv,
		  const Coord &which,
		  unsigned long long offset, int n,
		  std::vector<cacheloc_ent> &list, unsigned sz);

	 bool get_rand_nodes( wfs_file_id fid, wfs_version v, 
		  unsigned long long offset, int n, 
		  std::vector<cacheloc_ent> &list, unsigned sz);

	 void update_cacheloc( wfs_file_id fid, wfs_version v, 
		  std::vector<cacheloc_ent> const &list );

  protected:
	 std::map<uint64_t, std::vector<cacheloc_ent>* > _locs;
  private:
	 pthread_mutex_t _writes_lock;
};

#endif /* __WFS_CACHELOC_H */

