/**
 * Copyright (c) 2009 Jeremy Stribling, Yair Sovran, Irene Zhang,
 *                    Xavid Pretzer, Jinyang Li, Frans Kaashoek, Robert Morris,
 *     		      Massachusetts Institute of Technology,
 * 		      New York University
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */


#include <arpa/inet.h>
#include <chash.h>
#include <host.h>
#include <method_thread.h>
#include "host_marshall.h"
#include "node_list_simple.h"
#include "node_manager_cent.h"
#include "wfs_cent_cserver.h"
#include <wfs_misc.h>

node_manager_cent::node_manager_cent( wfs_node me, sockaddr_in cserver,
				      int ping_period, vivaldi *v,
				      wfs_client_manager *clients )
  : _me(me), _cserver(cserver), _viv(v), _clients(clients),
    _ping_period(ping_period) {

  _nodes = new node_list_simple( _me );
  _cserver_node = make_node( inet_ntoa(_cserver.sin_addr),
			     htons(_cserver.sin_port) );
  method_thread( this, true, &node_manager_cent::loop );

}

node_manager_cent::~node_manager_cent() {
  delete _nodes;
}

void node_manager_cent::loop() {

  sleep(1);

  while(1) {

    std::vector<host> x;
    rpcc *client = _clients->get_client( _cserver_node );
    if( client == NULL ) {
      sleep(_ping_period);
      continue;
    }
    client->call( wcc_protocol::fetch, 0, x );
    _clients->done_with_client(_cserver_node);

    std::vector<wfs_node> wnodes;

    // TODO: this is oh so inefficient
    // TODO: Also, we probably need to generate a view number somehow,
    //  and attach it to all wfs RPCs.

    wfs_log(WFS_DBG_1,"received these hosts: " );

    for( unsigned i = 0; i < x.size(); i++ ) {

      wfs_log(WFS_DBG_1, "%s:%d_%s ", inet_ntoa( x[i].sin().sin_addr ),
		htons(x[i].port), x[i].site_id.c_str() );

      wnodes.push_back( make_node( inet_ntoa( x[i].sin().sin_addr ),
				   htons( x[i].port ), x[i].site_id ) );
    }


    wfs_log(WFS_DBG_1,"\n" );



    wfs_log(WFS_DBG_1,"wfs_client my current vivaldi %s info %s\n",
	     _viv?(_viv->my_coords()).print_coord().c_str():"null",
	     _viv?_viv->print_allcoords().c_str():"empty");

    _nodes->sync_to_node_list( wnodes );

    sleep(_ping_period);

  }

}
