/**
 * Copyright (c) 2009 Jeremy Stribling, Yair Sovran, Irene Zhang, 
 *                    Xavid Pretzer, Jinyang Li, Frans Kaashoek, Robert Morris,
 *     		      Massachusetts Institute of Technology,
 * 		      New York University
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */


#ifndef paxos_h
#define paxos_h

#include <string>
#include <vector>
#include "rpc.h"
#include "paxos_protocol.h"
#include "paxos_log.h"
#include "wfs_misc.h"
#include <wfs_client_manager.h>

class paxos_view_change {
 public:
  virtual void view_change() = 0;
  virtual ~paxos_view_change() {};
};

class paxos {
 private:
  paxos_log *l;
  rpcs *pxs;
  wfs_client_manager *_clients;

  pthread_mutex_t pxs_mutex;
  pthread_cond_t manager_cond;
  pthread_cond_t heartbeat_cond;

  unsigned c_vid;
  bool stable;
  time_t time_since_unstable;
  std::string me;
  paxos_view_change *vc;
  std::vector<std::string> c_mems;

  prop_t my_n;
  prop_t n_h;
  prop_t n_a;
  std::vector<std::string> v_a;
  unsigned vid_h;
  std::map<unsigned,std::vector<std::string> > views;
  int delay_max;
  bool delay;

  void switchview(unsigned vid, std::vector<std::string> mems);
  void startvc();
  bool prepare(unsigned vid, std::vector<std::string> &accepts, 
         std::vector<std::string> mems,
         std::vector<std::string> &v);
  void accept(unsigned vid, std::vector<std::string> &accepts,
	      std::vector<std::string> &rejects, 
        std::vector<std::string> mems, std::vector<std::string> v);
  void decide(unsigned vid, std::vector<std::string> accepts,
        std::vector<std::string> v);
  void dodelay();
  void change(std::string m);
  paxos_protocol::status preparereq(std::string src, 
          paxos_protocol::preparearg a,
          paxos_protocol::prepareres &r);
  paxos_protocol::status acceptreq(std::string src, 
          paxos_protocol::acceptarg a,
          int &r);
  paxos_protocol::status decidereq(std::string src, 
          paxos_protocol::decidearg a,
          int &r);
  paxos_protocol::status heartbeat(std::string m, unsigned vid, int &r);

  bool isamember(std::string m, const std::vector<std::string> &mems);
  bool majority(const std::vector<std::string> &l1, const std::vector<std::string> &l2);
  rpcc *get_cl1(std::string m);

  friend class paxos_log;
 public:
  paxos(std::string _first, std::string _me, std::string log_dir,
	wfs_client_manager *clients, int delay_max = 10,
	paxos_view_change *vc = 0);
  ~paxos() {};

  void get_members(std::vector<std::string> &m);
  std::string myaddr() { return me; };
  unsigned vid() { return c_vid; }
  bool is_stable() { return stable; };
  std::string print_members(const std::vector<std::string> &m);
  rpcc *get_cl(std::string m);
  void return_cl(std::string m);
  rpcs *get_rpcs() { return pxs; };

  void set_paxos_view_change(paxos_view_change *v) { vc = v; }

  void start_paxos();
  void manager(void);
  void heartbeater(void);
  void doheartbeat(std::string m);
};

#endif /* paxos_h */
