/**
 * Copyright (c) 2009 Jeremy Stribling, Yair Sovran, Irene Zhang, 
 *                    Xavid Pretzer, Jinyang Li, Frans Kaashoek, Robert Morris,
 *     		      Massachusetts Institute of Technology,
 * 		      New York University
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */


/*
 * slice_lock.cc
 */
#include "slice_lock.h"

using namespace std;

slice_lock::slice_lock(const slice_lock & sl)
{
  name = sl.name;
  avail= sl.avail;
  state = sl.state;
  lease_period = sl.lease_period;
  granted_on = sl.granted_on;
  primary = sl.primary;
  last_modified = sl.last_modified;
}

// For debugging
const std::string
slice_lock::toString() const
{
  ostringstream ost;
  string r;
  ost << "Slice-ID:" << std::hex << name << std::dec << std::endl;
  switch (avail) {
  case FREE:
    ost << "\tFree" << std::endl;
    break;
  case OWNED:
    ost << "\tOwned by:" << primary << " For " << lease_period << " Seconds. Expires in: ";
    (ttexpire() > 0) ? (ost << ttexpire()) : (ost << "Unknown/Already Expired"); ost << std::endl;
    ost << "\tOpaque state:" << state;
    break;
  }
  r = ost.str();
  return r;
}

std::ostream &
operator<<(std::ostream &os, const slice_lock &l)
{
  // We can't convert the host directly because we lose the site id. Also we don't log when was it granted.
  os << l.name << " " << static_cast<int>(l.avail) << " " << l.lease_period << " ";
  if(l.avail == slice_lock::OWNED){
    os << l.primary.addr << " " << l.primary.port << " " <<  l.primary.site_id << " ";
  } else {
    os << "NA" << " ";
  }
  os << l.state.length() << " ";
  os << l.state << endl;
  //os.write(l.state.c_str(),l.state.length());

  return os;
}

std::istream &
operator>>(std::istream &is, slice_lock &l)
{
  int i;
  unsigned int addr,port;
  std::string site;
  is >> l.name;
  is >> i;
  l.avail = static_cast<slice_lock::lavail>(i);
  is >> l.lease_period;
  //is >> l.granted_on; // Dont log this anymore, since we don't replicate renewal.
  if(l.avail == slice_lock::OWNED){
    is >> addr; is >> port; is >> site;
    l.primary = host(addr,port,site); // TODO: When host map is enabled, check the map before instantiation.
  } else {
    std::string tmp;
    is >> tmp;
    assert(tmp == "NA");
  }
  unsigned int len;
  is >> len;
  char space;
  is.get(space);
  char * buf = new char[len+1];
  is.getline(buf,len+1,'\n');
  buf[len] = '\0';
  l.state = buf;
  delete[] buf;
  return is;
}
// Use the string marshaling method.
marshall & operator<<(marshall &m, slice_lock l)
{
  ostringstream ost;
  ost << l;
  m << ost.str();
  return m;
}
unmarshall & operator>>(unmarshall &u, slice_lock &l)
{
  string s;
  u >> s;
  istringstream ist(s);
  ist >> l;
  return u;
}


