/**
 * Copyright (c) 2009 Jeremy Stribling, Yair Sovran, Irene Zhang, 
 *                    Xavid Pretzer, Jinyang Li, Frans Kaashoek, Robert Morris,
 *     		      Massachusetts Institute of Technology,
 * 		      New York University
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */


#ifndef __WFS_BT_CLIENT_H__ 
#define __WFS_BT_CLIENT_H__ 1

#include <ext/hash_map>
#include "wfs_cache_client.h"
#include "wfs_cacheloc.h"

using __gnu_cxx::hash_map;

class wfs_bt_client : public wfs_cache_client {

 protected:
  wfs_cacheloc _cache_loc; 
  sockaddr_in _bt_addr;
  hash_map<wfs_file_id, time_t> _fetched_at_times;
  hash_map<wfs_file_id, hash_map<uint32_t, int> > _nodes_in_use;
  pthread_mutex_t _nodes_in_use_lock;

  hash_map<wfs_file_id, std::vector<int32_t> > _my_block_map;
  pthread_mutex_t _my_block_map_lock;

  typedef struct add_cache_struct {
    wfs_node file_node;
    wfs_file_handle fh;
    wfs_version version;
    unsigned long long off;
    unsigned long long n;
  } add_cache_struct;

  std::vector<add_cache_struct> _acs_queue;
  pthread_mutex_t _acs_lock;
  pthread_cond_t _acs_condvar;

  void add_cache_loc();

 public:
  const static unsigned ASK_LIST_SIZE;
  wfs_bt_client( sockaddr_in bt_addr, wfs_node me, slice_manager *slices, 
		 wfs_cache *cache, rpcs *rpc_server,
		 vivaldi *viv, wfs_client_manager *clients, 
		 bool read_ahead, int prefetch_threads );
  
  wfs_protocol::status readcachedfile(  const std::string &user,
					wfs_file_handle fh, wfs_version version, 
				       unsigned long long off, unsigned int n, 
				       wfs_protocol::readinfo &info );
  
  wfs_protocol::status read_uncached_data( wfs_file_handle fh, 
					   wfs_version version, 
					   unsigned long long off, 
					   unsigned int n,
					   const std::string &user,
					   std::string &buf );

};

#endif /* __WFS_BT_CLIENT_H_ */
