/**
 * Copyright (c) 2009 Jeremy Stribling, Yair Sovran, Irene Zhang, 
 *                    Xavid Pretzer, Jinyang Li, Frans Kaashoek, Robert Morris,
 *     		      Massachusetts Institute of Technology,
 * 		      New York University
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */


#ifndef host_h
#define host_h

#include <vector>
#include <strings.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <ostream>

struct host {
  host(unsigned int a, unsigned int p, std::string s = "")
  { addr = a; port = p; site_id = s; }
  host(const char *hname, const char *pname, std::string site_id = "");
  host() { addr = 0; port = 0; site_id = ""; }
  host(const host &);
  sockaddr_in sin() {
    sockaddr_in s;
    bzero(&s, sizeof(s));
    s.sin_family = AF_INET;
    s.sin_addr.s_addr = addr;
    s.sin_port = port;
    return s;
  };
  unsigned int addr;
  unsigned int port;
  std::string site_id;

  operator std::string();
  const std::string str() const;
};

inline bool operator==(const host &a, const host &b) {
  return (a.addr == b.addr) && (a.port == b.port);
}

inline bool operator!=(const host &a, const host &b) {
  return (a.addr != b.addr) || (a.port != b.port);
}

inline bool operator>(const host &a, const host &b) {
  return (ntohl(a.addr) > ntohl(b.addr)) ||
         (a.addr == b.addr && ntohs(a.port) > ntohs(b.port));
}

inline bool operator<(const host &a, const host &b) {
  return (ntohl(a.addr) < ntohl(b.addr)) ||
         (a.addr == b.addr && ntohs(a.port) < ntohs(b.port));
}

std::ostream & operator<<(std::ostream &os, const host h);
std::ostream & operator<<(std::ostream &os, const std::vector<host> v);
bool same_hosts(std::vector<host> a, std::vector<host> b);
bool in_hosts(host h, std::vector<host> v);
std::vector<host> host_union(std::vector<host> a, std::vector<host> b);

#endif
