/**
 * Copyright (c) 2009 Jeremy Stribling, Yair Sovran, Irene Zhang, 
 *                    Xavid Pretzer, Jinyang Li, Frans Kaashoek, Robert Morris,
 *     		      Massachusetts Institute of Technology,
 * 		      New York University
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */


#ifndef __WFS_PERFILE_LOCKER_H__
#define __WFS_PERFILE_LOCKER_H__ 1

#include "wfs_file.h"

#include <assert.h>
#include <ext/hash_map>
#include <pthread.h>
#include <vector>
using __gnu_cxx::hash_map;

class wfs_perfile_locker {

 private:

  typedef struct lock_struct {

    pthread_mutex_t *lock;
    int count;

    lock_struct() {
      lock = new pthread_mutex_t();
      assert( pthread_mutex_init( lock, NULL ) == 0 );
      count = 0;
    }

    ~lock_struct() {
      pthread_mutex_destroy( lock );
      delete lock;
    }

  } lock_struct;

  pthread_mutex_t _locks_mutex;
  hash_map<wfs_file_id, lock_struct *> _locks;
  std::vector<lock_struct *> _new_locks;
  pthread_mutex_t _stale_mutex;
  pthread_cond_t _stalewait_cv;
  hash_map<wfs_file_id, bool> _stale_map;
  hash_map<wfs_file_id, bool> _stalewait_map;

 public:

  wfs_perfile_locker();
  ~wfs_perfile_locker();
  void lock_file( wfs_file_handle fh );
  void unlock_file( wfs_file_handle fh );
  void mark_stale( wfs_file_handle fh, bool stale );
  bool is_stale( wfs_file_handle fh, int to_wait_ms = -1 );

};

#endif // __WFS_PERFILE_LOCKER_H__
