/**
 * Copyright (c) 2009 Jeremy Stribling, Yair Sovran, Irene Zhang, 
 *                    Xavid Pretzer, Jinyang Li, Frans Kaashoek, Robert Morris,
 *     		      Massachusetts Institute of Technology,
 * 		      New York University
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */


#ifndef __WFS_STORAGE_H__
#define __WFS_STORAGE_H__ 1

#include "wfs_metadata.h"

class wfs_storage {

 protected:
  std::string _storage_dir;
  bool _md_memory_only;
  bool _md_xattr;

  pthread_mutex_t _md_lock;
  std::map< wfs_file_id, wfs_metadata *> _metadata;

  std::string filename(wfs_file_handle, int32_t vernum = 0);
  virtual bool find_fid( void *dirp, const std::string &name, 
			 char *line, bool use_line, 
			 wfs_file_id &fid, bool &higher,
			 char **line_buf, int &line_size );
  virtual void * open_finder( wfs_file_handle fh );
  virtual void close_finder( void *dirp );
  virtual int read_from_finder( void *filep, char *buf, int size,
				const unsigned char **blockp );

 public:

  struct dirent {
    std::string name;
    wfs_file_handle inum;
  };

  wfs_storage( std::string dir, bool set = true );
  virtual ~wfs_storage();

  void set_dir( std::string dir, bool set = true );
  wfs_metadata *get_md( wfs_file_id fid, int32_t vernum = 0 );
  void set_and_release_md( wfs_file_id fid, wfs_metadata *md );
  void release_md( wfs_metadata *md );

  bool create_new_object( wfs_file_handle fh, int32_t vernum = 0 );
  bool delete_object( wfs_file_handle fh, int32_t vernum = 0 );
  bool object_exists( wfs_file_handle fh, int32_t vernum = 0 );
  bool stat_object( wfs_file_handle fh, struct stat *sb, int32_t vernum = 0 );
  bool set_stat_object( wfs_file_handle fh, struct stat *sb, 
			int32_t vernum = 0 );
  bool link( wfs_file_handle dh, std::string name, 
	     wfs_file_handle new_fh, std::string &append_data );
  bool lookup( wfs_file_handle dh, std::string name, wfs_file_id &fid );
  bool unlink( wfs_file_handle dh, std::string name );
  bool readdir( wfs_file_handle fh, unsigned long long ind, 
		unsigned int n, std::vector<dirent> &e );
  const void save_old_version( wfs_file_handle fh );
  virtual bool readfile( wfs_file_handle fh, wfs_version version, 
			 unsigned long long off, int n, std::string &buf );
  virtual bool writefile( wfs_file_handle fh, 
			  std::vector<unsigned long long> off, 
			  std::vector<std::string> buf );
  bool truncate( wfs_file_handle fh, unsigned long long off );
  unsigned long long file_checksum( wfs_file_handle fh );

};

#endif // __WFS_STORAGE_H
